/**
 * @file       adc_single_chnl_exmple.c
 * @version    1.0
 * @date       03 July 2019
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief   This file contains example describing usage of ADC APIs.
 * @section Description
 * Conversion of Analog signal to Digital samples with 100Ksps as sampling rate.
 */

/*Includes*/
#include "rsi_chip.h"
#include "aux_reference_volt_config.h"
#include "rsi_adc.h"
#include "rsi_bod.h"
#include "rsi_board.h"


/* Private define ------------------------------------------------------------*/ 
#define CORE_CLOCK_180MHZ        1         /* This define use for enable core clock to 180Mhz ,
CORE_CLOCK_180MHZ : 1 ; Core clock is 180Mhz SOC PLL
CORE_CLOCK_180MHZ : 0 ; Core clock is 32Mhz RC
*/																								 
#define NUMBER_OF_CHA_EN         1          /* Single channel data acquisition */  
#define CHANNEL0_SAMPLE_LENGTH   977        /* Number of ADC sample collect for operation*/  
#define FIR_FILTER_ENABLE        1          /* Filter enable on ADC output ,If filter enable then max number of samples can filter=1023-(Number of coeff *2)*/
#define FINDOUT_INPUT_VOLTAGE    1          /* Enable to print the equivalent input voltage */ 
float   vref_value = 2.8;                   /* Reference voltage to ADC for conversion operation */  

/* Private variables ---------------------------------------------------------*/
volatile uint8_t chnl0_complete_flag = 0;

/* Private output buffer ---------------------------------------------------------*/
int16_t adc_output_ch0[CHANNEL0_SAMPLE_LENGTH];

static uint8_t first_flag = 1;

#if FINDOUT_INPUT_VOLTAGE
float adc_input_eqv_vltg[CHANNEL0_SAMPLE_LENGTH];
#endif

/* FIR Filter related parameter */
#if FIR_FILTER_ENABLE
q15_t filerStateq15[1];
arm_fir_instance_q15_opt S15;
volatile uint8_t fim_intr = 0;

/* Maximum sampling rate achived */
#define FILTER_COEFF_23_Q15        1       /* If this MACRO enable then FIR filter coefficient is 23 */
#define FILTER_COEFF_88_Q15        0      /* If this MACRO enable then FIR filter coefficient is 88  */

#if FILTER_COEFF_23_Q15
#include "coeff_1dB_20dB_300Khz_5mhz_q15.c"
#define   NUM_TAPS            23		                                
#endif

#if FILTER_COEFF_88_Q15
#include "1dB_20dB_25Khz_75khz_5mhz_q15.c"
#define   NUM_TAPS            88			
#endif
int16_t temp_buffer[CHANNEL0_SAMPLE_LENGTH + NUM_TAPS];
int32_t fir_result1[CHANNEL0_SAMPLE_LENGTH*1];
#endif


/* Core clock modification APIs */
#if CORE_CLOCK_180MHZ
extern void rsi_delay_ms(uint32_t val);
#define SOC_PLL_REF_FREQUENCY           32000000  /*<! PLL input REFERENCE clock 32MHZ */
//Note: Change this macro to required PLL frequency in hertz
#define PS4_SOC_FREQ                    180000000 /*<! PLL out clock 180MHz            */
/*Pre-fetch and regestring */
#define ICACHE2_ADDR_TRANSLATE_1_REG  *(volatile uint32_t *)(0x20280000 + 0x24)
#define MISC_CFG_SRAM_REDUNDANCY_CTRL *(volatile uint32_t *)(0x46008000 + 0x18)
#define MISC_CONFIG_MISC_CTRL1        *(volatile uint32_t *)(0x46008000 + 0x44)
#define SWITCH_QSPI_TO_SOC_PLL

/**
 * @brief  This function is callback function to reconfigure ping and pong address.
 * @param  channel_no  :  Channel number
 * @param  event       :  Event type 
 * @retval None
 */
void soc_pll_config(void)
{	
	/*Default keep M4 in reference clock*/
	RSI_CLK_M4SocClkConfig(M4CLK,M4_ULPREFCLK,0);
#if (PS4_SOC_FREQ >= 120000000)
	/*Configure the prefetch and registering when SOC clock is more than 120Mhz*/
	/*Configure the SOC PLL to 220MHz*/
	ICACHE2_ADDR_TRANSLATE_1_REG  = BIT(21);  //Icache registering when clk freq more than 120
	/*When set, enables registering in M4-TA AHB2AHB. This will have performance penalty. This has to be set above 100MHz*/
	MISC_CFG_SRAM_REDUNDANCY_CTRL = BIT(4);
	MISC_CONFIG_MISC_CTRL1 |= BIT(4); //Enable Register ROM as clock frequency is 200 Mhz
#endif	
	/*Configure the PLL frequency*/
	RSI_CLK_SetSocPllFreq(M4CLK , PS4_SOC_FREQ , SOC_PLL_REF_FREQUENCY);

	/*Switch M4 clock to PLL clock for speed operations*/
	RSI_CLK_M4SocClkConfig(M4CLK,M4_SOCPLLCLK,0);
#ifdef SWITCH_QSPI_TO_SOC_PLL
	ROMAPI_M4SS_CLK_API->clk_qspi_clk_config(M4CLK,QSPI_SOCPLLCLK,0,0,0); 
#endif
	/* Switch ULP Pro clock to 90 MHZ */
	RSI_ULPSS_ClockConfig(M4CLK,1,1,0);

	RSI_ULPSS_UlpProcClkConfig(ULPCLK,ULP_PROC_SOC_CLK,0,&rsi_delay_ms);	
}	
#endif

/* FIR Filter related APIs */
#if FIR_FILTER_ENABLE	
void adc_prepostprocessing_to_fir()
{
	uint16_t j=0;	
	for(j = 0; j < (CHANNEL0_SAMPLE_LENGTH + NUM_TAPS); j++)
	{
		if(j < NUM_TAPS)
		{
			temp_buffer[j] = temp_buffer[CHANNEL0_SAMPLE_LENGTH+j]; 		
		}
		else
		{
			temp_buffer[j] = adc_output_ch0[j-NUM_TAPS]<<4; 
		}				
	}

	/* Configure the FIR Filter parameter */
	arm_fir_init_q15_opt(&S15, NUM_TAPS , (q15_t *)&filterCoeffsq15_int,
			&filerStateq15[0],(CHANNEL0_SAMPLE_LENGTH + NUM_TAPS));	
	rsi_arm_fir_q15_opt(&S15,temp_buffer,adc_output_ch0,(CHANNEL0_SAMPLE_LENGTH + NUM_TAPS),
			BANK3,BANK4,BANK6);	

	/* Wait for FIR-Filter operation to be done */		
	while(!fim_intr);
	fim_intr = 0;				

	if(first_flag)
	{	
		fim_read_data((BANK6+(NUM_TAPS*2)),(CHANNEL0_SAMPLE_LENGTH-NUM_TAPS),
				(adc_output_ch0),FORMAT_Q15 ,ULP_FIM_COP_DATA_REAL_REAL);				
		first_flag = 0; 
	}
	else
	{
		fim_read_data((BANK6+(NUM_TAPS*1)),(CHANNEL0_SAMPLE_LENGTH),
				(adc_output_ch0),FORMAT_Q15 ,ULP_FIM_COP_DATA_REAL_REAL);								
	}	  				
}
/**
 * @brief  This function is FIM IRQ to clear the FIM interrupt.
 * @param  None
 * @retval None
 */
void IRQ017_Handler(void)
{	
	fim_interrupt_handler(FIM);	
	fim_intr = 1;
} 
#endif

/**
 * @brief  Call back function , This function interrupt handler  
 * @param  channel_no  : Channel number for clearing   
 * @retval None
 */
void callback_handler(uint8_t channel_no,uint8_t event)
{	
	if(event == INTERNAL_DMA)
	{
		chnl0_complete_flag = 1;
	}
}


/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */
int main()
{  
	/* At this stage the MICROCONTROLLER clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs1xxxx.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs1xxxx.c file
	 */
	uint32_t status = 0,i;
	float battery_status = 0,vout = 0,max_ip_volt_scdc = 2.4;

	adc_ch_config_t adcChConfig;
	adc_config_t    adcConfig;

	SystemCoreClockUpdate();	

	/* Read the input supply of chip */
	battery_status = RSI_BOD_SoftTriggerGetBatteryStatus();

	/*If input to chip less than 2.4V then switch input voltage supply from SCDC to HPLDO */ 	
	if(battery_status < max_ip_volt_scdc)
	{
		RSI_IPMU_ProgramConfigData(hp_ldo_voltsel);		
	}	

#if CORE_CLOCK_180MHZ	
	/* Increase M4 Clock */
	soc_pll_config();
#endif

#if FIR_FILTER_ENABLE	
	/*Fim clock enable */
	RSI_ULPSS_PeripheralEnable(ULPCLK,ULP_FIM_CLK,ENABLE_STATIC_CLK);

	NVIC_EnableIRQ(FIM_IRQn);
#endif

	/* Configure ADC module related parameter */
	adcConfig.num_of_channel_enable = NUMBER_OF_CHA_EN;	     /* Number of channel enable for ADC operation,here enable two ADC channel */	
	adcConfig.operation_mode        = ADC_FIFOMODE_ENABLE;	 /* ADC operation mode */

	/* Configure ADC channel related parameter */
	adcChConfig.input_type[0] 		   = 0;            					 /* Single ended input to ADC */ 
	adcChConfig.sampling_rate[0]     = 100000;      					 /* Sampling rate value ,Maximum value is 2.5msps (2500000)*/
	adcChConfig.pos_inp_sel[0] 	     = 2;        							 /* Positive input to ADC from ULP_GPIO4 */
	adcChConfig.neg_inp_sel[0] 	     = 2;        							 /* Negative input to ADC from ULP_GPIO5,For single ended data acquisition this parameter not used */ 
	adcChConfig.opamp_gain[0] 	     = 2;         						 /* OPAMP gain value if opamp output is one of input to ADC*/
	adcChConfig.rx_buf[0] 	         = adc_output_ch0;         /* Output buffer */
	adcChConfig.num_of_samples[0]    = CHANNEL0_SAMPLE_LENGTH; /* Number of ADC samples collect for operation */ 


	/* Configure the Verf value ADC calibration and enable ADC power gate)*/
	status = ADC_Init(adcChConfig,adcConfig,callback_handler);

	if(status)
	{
		/* Configure proper sampling rate */ 
		while(1);
	}

	/* Configure reference voltage for analog peripheral ,here till 2.8V generate by using 
	   AUX_LDO so more than 2.8V enable LDO bypass mode */	
	RSI_AUX_RefVoltageConfig(vref_value,battery_status);

	ADC_ChannelConfig(adcChConfig,adcConfig);

	/* Start the ADC block */
	ADC_Start();

	while(1)
	{				
		while(!chnl0_complete_flag);
		chnl0_complete_flag = 0;

#if FIR_FILTER_ENABLE
		adc_prepostprocessing_to_fir();			
#endif	

#if FINDOUT_INPUT_VOLTAGE		
		for(i = 0; i < CHANNEL0_SAMPLE_LENGTH; i++)
		{
#if FIR_FILTER_ENABLE
			if(adc_output_ch0[i] & BIT(15))
			{
				fir_result1[i] = (adc_output_ch0[i] & (0x7FFF));
			}	
			else
			{
				fir_result1[i] = adc_output_ch0[i] | BIT(15);	
			}			

			vout = (( (float) fir_result1[i] / (float) 65536 ) * vref_value ); 
#else
			if(adc_output_ch0[i] & BIT(11))
			{
				adc_output_ch0[i] = (adc_output_ch0[i] & (ADC_MASK_VALUE));
			}	
			else
			{
				adc_output_ch0[i] = adc_output_ch0[i] | BIT(11);	
			}			

			vout = (( (float) adc_output_ch0[i] / (float) 4096 ) * vref_value ); 			
#endif
			if(adcChConfig.input_type[0])
			{
				vout = vout - (vref_value/2);			
			}
			adc_input_eqv_vltg[i] = vout;     			
		}
#endif		
	}
}	
