/**
 * @file       adc_static_mode_exmp.c
 * @version    1.0
 * @date       03 July 2019
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief   This file contains example describing usage of ADC static mode.
 * @section Description
 * Conversion of Analog signal to Digital samples with 1Msps sampling rate using ADC in static ,
 * No interrupt available in ADC static mode so after two ADC cycle read the ADC sample.
 * In ADC static mode we can collect only one sample at a time.
 */

/*Includes*/
#include "rsi_chip.h"
#include "aux_reference_volt_config.h"
#include "rsi_adc.h"
#include "rsi_bod.h"
#include "rsi_board.h"

/* Private define ------------------------------------------------------------*/ 
#define NUMBER_OF_CHA_EN          1         /* In ADC static mode enable only one ADC channel */  
#define CHANNEL0_SAMPLE_LENGTH    1         /* Number of sample to collect  */   
float   vref_value = 2.8; /* Reference voltage to ADC for conversion operation */ 

/* Private variables ---------------------------------------------------------*/
volatile uint8_t intr_done = 0;

/* Private output buffer ---------------------------------------------------------*/
int16_t adc_output_ch0[CHANNEL0_SAMPLE_LENGTH];		/* ADC output buffer */

/**
 * @brief  Call back function , This function interrupt handler  
 * @param  channel_no  : Channel number for clearing   
 * @retval None
 */
void callback_handler(uint8_t channel_no,uint8_t event)
{	
	/* Call back handler never call in ADC static mode */ 
}

/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */
int main()
{  
	/* At this stage the MICROCONTROLLER clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs1xxxx.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs1xxxx.c file
	 */
	uint32_t status = 0;
	volatile float  vout;
	float battery_status;
	float  max_ip_volt_scdc = 2.4;

	/* ADC Channel configuration structure variable */
	adc_ch_config_t adcChConfig;
	/* ADC module configuration structure variable */
	adc_config_t    adcConfig;

	SystemCoreClockUpdate();	

	/* Read the input supply of chip */
	battery_status = RSI_BOD_SoftTriggerGetBatteryStatus();

	/*If input to chip less than 2.4V then swith input voltage supply from SCDC to HPLDO */ 
	if(battery_status < max_ip_volt_scdc)
	{
		RSI_IPMU_ProgramConfigData(hp_ldo_voltsel);		
	}	

	DEBUGINIT();

	/* Configure ADC module related parameter */
	adcConfig.num_of_channel_enable = NUMBER_OF_CHA_EN;	     /* Number of channel enable for ADC operation,here enable two ADC channel */
	adcConfig.operation_mode        = ADC_STATICMODE_ENABLE;	 /* ADC operation mode */

	/* Configure ADC channel related parameter */
	adcChConfig.input_type[0] 		   = 0;        /* Single ended input to ADC */ 
	adcChConfig.sampling_rate[0]     = 1000000;  /* Sampling rate value ,Maximum value is 2.5msps (2500000)*/
	adcChConfig.pos_inp_sel[0] 	     = 2;        /* Positive input to ADC from ULP_GPIO4 */
	adcChConfig.neg_inp_sel[0] 	     = 2;        /* Negative input to ADC from ULP_GPIO5,For single ended data aqcquisition this parameter not used */  
	adcChConfig.opamp_gain[0] 	     = 0;         /* OPAMP gain value if opamp output is one of input to ADC*/ 
	adcChConfig.rx_buf[0] 	         = adc_output_ch0;         /* Output buffer */
	adcChConfig.num_of_samples[0]    = CHANNEL0_SAMPLE_LENGTH; /* Number of ADC samples collect for operation */ 

	/* Configure the Verf value ADC calibration and enable ADC power gate)*/
	status = ADC_Init(adcChConfig,adcConfig,callback_handler);

	if(status)
	{
		/* Configure proper sampling rate */ 
		while(1);
	}

	/* Configure reference voltage for analog peripheral ,here till 2.8V generate by using 
	   AUX_LDO so more than 2.8V enable LDO bypass mode */
	RSI_AUX_RefVoltageConfig(vref_value,battery_status); 

	ADC_ChannelConfig(adcChConfig,adcConfig);

	/* Start the ADC block */
	ADC_Start();

	while(1)
	{
		/* wait for two ADC cycle before reading ADC sample,here ADC sampling rate=1M so 1 ADC Cycle=1us*/ 
		adc_output_ch0[0] = RSI_ADC_ReadDataStatic(AUX_ADC_DAC_COMP,1,adcChConfig.input_type[0]);

		if (adc_output_ch0[0] & BIT(11))
		{
			adc_output_ch0[0] = (adc_output_ch0[0] & (ADC_MASK_VALUE));
		}
		else
		{
			adc_output_ch0[0] = adc_output_ch0[0] | BIT(11);
		}	

		vout = (( (float) adc_output_ch0[0] / (float) 4096 ) * vref_value ); 

		if(adcChConfig.input_type[0])
		{
			vout =vout - (vref_value/2);
			DEBUGOUT("Differential ended input  :%f\n",vout);					
		}
		else
		{				
			DEBUGOUT("Single ended input :%f\n",vout);	
		}
	}
}	
