/**
 * @file       rsi_comparator_example.c
 * @version    1.0
 * @date       4 Apr 2019
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This file contains example describing usage of comaparator APIs.
 *
 * @section Description
 * This example demonstrate simple application of comparator , here for comparator1 are used
 * for comparing voltage on AGPIO0 and AGPIO1
 */
 
 /* Includes ------------------------------------------------------------------*/
#include "rsi_chip.h"

/* Example Selection MACROs -----------------------------------------------------------*/
#define COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_EXTERNAL         1
#define COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_REF_SCALER       0

/* Private define ------------------------------------------------------------*/ 
#define COMP1_INTERRUPT_HANDLER IRQ008_Handler /* Comparator1 IRQ handler number */
#define HYST_ENABLE             0  
#define FILTER_ENABLE           0 
#define NPSS_GPIO_PIN           3

#if COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_EXTERNAL
		#define COMP1_POSITIVE_INPUT    0 /* Select the ULP_GPIO0 for comparator non-inverting input */   
		#define COMP1_NEGATIVE_INPUT    0 /* Select the ULP_GPIO1 for comparator inverting input */ 
#endif

#if COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_REF_SCALER
		#define COMP1_POSITIVE_INPUT    0  /* Select the ULP_GPIO0 for comparator non-inverting input */   
		#define COMP1_NEGATIVE_INPUT    4  /* Select the reference scaler output for comparator inverting input */ 
		#define SCALER_FACT_VAL         7  /* Reference scaler output is 0.8V */   
#endif

/* Private variable */
float vref_value = 2.4;

/**
 * @brief  This function is comparator IRQ handler , its disable the comparator IRQ.
 * @param  None
 * @retval None
 */
void COMP1_INTERRUPT_HANDLER(void)
{
	/* Toggle the UULP_GPIO3 for comparator interrupt indication */
	RSI_NPSSGPIO_SetPin(NPSS_GPIO_PIN , 1);
	RSI_NPSSGPIO_SetPin(NPSS_GPIO_PIN , 0);

	/* Mask the comparator interrupt */
	RSI_COMP_OutputMaskConfig(COMP1,ENABLE);
	/* UnMask the comparator interrupt */
	RSI_COMP_OutputMaskConfig(COMP1,DISABLE);	
}

 

/**
 * @brief  Main program.
 * @param  None
 * @retval None
*/
int main()	
{
	/* At this stage the MICROCONTROLLER clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs1xxxx.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs1xxxx.c file
	*/
	
	/*Configures the system default clock and power configurations*/
 	SystemCoreClockUpdate();
	
	/* Configure the UULP_GPIO3 for comparator interrupt indication */
	RSI_NPSSGPIO_InputBufferEn(NPSS_GPIO_PIN , 1U);
	RSI_NPSSGPIO_SetPinMux(NPSS_GPIO_PIN , 0);
	RSI_NPSSGPIO_SetDir(NPSS_GPIO_PIN , NPSS_GPIO_DIR_OUTPUT);		
  
	/* Configure the AUX clock source */
  RSI_ULPSS_AuxClkConfig(ULPCLK, ENABLE_STATIC_CLK ,ULP_AUX_32MHZ_RC_CLK);	
	
	/*  Configure the reference LDO voltage as 2.48v */	 	
	RSI_AUX_RefVoltageConfig(vref_value,3.2);
 
  #if COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_EXTERNAL
	/* Configure the comparator parameters,Here non-inverting input 
	   configured for ULP_GPIO0 and inverting input configured for ULP_GPIO1 */
	RSI_COMP_Config(AUX_ADC_DAC_COMP,COMP1,COMP1_POSITIVE_INPUT,COMP1_NEGATIVE_INPUT,
	                HYST_ENABLE,FILTER_ENABLE);  	
	#endif
  
  #if COMP1_POS_INPUT_EXTERNAL_NEG_INPUT_REF_SCALER
  /* Configure the comparator parameters */
	RSI_COMP_Config(AUX_ADC_DAC_COMP,COMP1,COMP1_POSITIVE_INPUT,COMP1_NEGATIVE_INPUT,
	                HYST_ENABLE,FILTER_ENABLE);	
	
	/* Set the reference scaler factor value set as 7,
	   so inverting terminal input is 0.8V*/
	RSI_COMP_ReferenceScaler(AUX_ADC_DAC_COMP,SCALER_FACT_VAL);
  #endif
	
  /* enable  the comparator */
	RSI_COMP_Enable(AUX_ADC_DAC_COMP,COMP1,ENABLE);
	
	while(1);		
}	
