/**
 * @file       rsi_qspi.h
 * @version    0.9.3
 * @date       6th March,2017
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This files contains functions prototypes related to power save
 *
 * @section Description
 * This file contains the list of function prototypes for the Power save and low level function definitions
 * Following are list of API's which need to be defined in this file.
 *
 */


/**
 * Includes
 */

#ifndef  __RSI_QSPI_H__
#define  __RSI_QSPI_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rsi_packing.h"
#include "stdint.h"

#define SINGLE_MODE                  0 	//! SPI mode
#define DUAL_MODE                    1	//! dual mode
#define QUAD_MODE                    2	//! quad mode
#define OCTA_MODE                    3
#define EN_PREFETCH                  1 	//! prefetch will be enabled
#define DIS_PREFETCH                 0	//! prefetch will be disabled
#define DUMMY_READS                  0	//! dummy's are read
#define DUMMY_WRITES                 1	//! dummy's are written
#define EN_EXTRA_BYTE										//! Extra byte will be enabled
#define DIS_EXTRA_BYTE									//! Extra byte will be disabled
#define CONTINUOUS                   1	//! continuous mode is selected
#define DIS_CONTINUOUS               0 	//! discontinuous mode is selected
#define SST_SPI_FLASH                0	//! sst spi flash
#define SST_DUAL_FLASH               1	//! sst dual flash
#define SST_QUAD_FLASH               2	//! sst quad flash
#define WBOND_QUAD_FLASH             3	//! Winbond quad flash
#define AT_QUAD_FLASH                4	//! Atmel quad flash
#define MX_QUAD_FLASH                5	//! macronix quad flash
#define EON_QUAD_FLASH               6	//! cFeon quad flash
#define MICRON_QUAD_FLASH            7	//! Micron quad flash
#define GIGA_DEVICE_FLASH            8	//! Giga Device flash
#define MX_OCTA_FLASH                9	//! macronix octa flash
#define ADESTO_OCTA_FLASH            10	//! Adesto octa flash
#define EN_AUTO_MODE                 1	//! Auto mode selection
#define EN_MANUAL_MODE               0	//! Manual mode selection
#define CHIP_ZERO                    0	//! cs-0
#define CHIP_ONE                     1	//! cs-1
#define CHIP_TWO                     2	//! cs-2
#define CHIP_THREE                   3	//! cs-3
#define EN_JUMP                      1	//! Enables jump
#define DIS_JUMP                     0	//! Disables jump
#define NEG_EDGE_SAMPLING            1	//! enables neg edge sampling
#define POS_EDGE_SAMPLING            0 	//! enables pos edge sampling
#define QSPI_FULL_TIME_CLK           1	//! full time clk will be provided
#define QSPI_DYNAMIC_CLK             0	//! dynamic clk gating will be enabled
#define EN_WR_PROT                   2	//! enable write protection
#define REM_WR_PROT                  1	//! remove write protection
#define DNT_REM_WR_PROT              0	//! no change to wr protection
#define DMA_MODE                     1	//! use dma only in manaul mode
#define NO_DMA                       0	//! dma will not be used
#define SWAP                         1 	//! swap will be enabled
#define NO_SWAP                      0	//! swap will be disabled
#define IGNORE_FULL_DUPLEX           2	//! do nothing for full duplex
#define EN_FULL_DUPLEX               1	//! enable full duplex
#define DIS_FULL_DUPLEX              0	//! disable full duplex
#define NO_WRAP                      7	//! wrap is diabled
#define SST_8BYTE_WRAP               0	//! 8 byte wrap will be used
#define SST_16BYTE_WRAP              1	//! 16 byte wrap will be used
#define SST_32BYTE_WRAP              2 	//! 32 byte wrap will be used
#define SST_64BYTE_WRAP              3	//! 64 byte wrap will be used
#define MICRON_16BYTE_WRAP           0	//! 16 byte wrap will be used
#define MICRON_32BYTE_WRAP           1 	//! 32 byte wrap will be used
#define MICRON_64BYTE_WRAP           2	//! 64 byte wrap will be used
#define _32BIT_ADDR                  4  //! 32 bit addr is configured
#define _24BIT_ADDR                  3  //! 24 bit addr is configured
#define _16BIT_ADDR                  2 	//! 16 bit addr is configured
#define _9BIT_ADDR                   1   //! 9 bit addr is configured
#define _8BIT_ADDR                   0  //! 8 bit addr is configured
#define MANUAL_DUMMY_BYTE_OR_BIT_MODE BIT(25)  
#define DUMMY_BYTE_OR_BIT_MODE       BIT(0)
#define CONTINUE_FETCH_EN            BIT(12)
#define WORD_SWAP_EN   20

//! This structure contains qspi registers
struct qspi_reg_s {
 volatile uint32_t QSPI_CLK_CONFIG_REG;            //!< qspi reg
 volatile uint32_t QSPI_BUS_MODE_REG;              //!< qspi reg
 volatile uint32_t QSPI_AUTO_CTRL_CONFIG_1_REG;    //!< qspi reg
 volatile uint32_t QSPI_AUTO_CTRL_CONFIG_2_REG;    //!< qspi reg
 volatile uint32_t QSPI_MANUAL_CONFIG_REG;         //!< qspi reg
 volatile uint32_t QSPI_MANUAL_CONFIG_2_REG;       //!< qspi reg
 volatile uint32_t RESERVED_1;   									 //!< qspi reg     
 volatile uint32_t QSPI_FIFO_THRESHOLD_REG;        //!< qspi reg
 volatile uint32_t QSPI_STATUS_REG;                //!< qspi reg
 volatile uint32_t QSPI_INTR_MASK_REG;             //!< qspi reg
 volatile uint32_t QSPI_INTR_UNMASK_REG;           //!< qspi reg
 volatile uint32_t QSPI_INTR_STS_REG;              //!< qspi reg
 volatile uint32_t QSPI_INTR_ACK_REG;              //!< qspi reg
 volatile uint32_t QSPI_STS_MC_REG;                //!< qspi reg
 volatile uint32_t QSPI_AUTO_CONFIG_1_CSN1_REG;    //!< qspi reg
 volatile uint32_t QSPI_AUTO_CONFIG_2_CSN1_REG;    //!< qspi reg
 volatile uint32_t QSPI_MANUAL_RD_WR_DATA_REG;     //!< qspi reg
 volatile uint32_t RESERVED_2[15];   							 //!< qspi reg     
 volatile uint32_t QSPI_MANUAL_WRITE_DATA_2_REG;   //!< qspi reg     
 volatile uint32_t RESERVED_3[3];   							 //!< qspi reg     
 volatile uint32_t QSPI_AUTO_CONFIG3;
 volatile uint32_t QSPI_AUTO_CONFIG3_CSN1;
 volatile uint32_t RESERVED_4[6];   							 //!< qspi reg     
 volatile uint32_t OCTA_SPI_BUS_CONTROLLER;   		 //!< qspi reg :2c
 volatile uint32_t QSPI_AUTO_BASE_ADDR_UNMASK_CSN0;
 volatile uint32_t RESERVED_7[3];   							 //!< qspi reg     
 volatile uint32_t OCTA_SPI_BUS_CONTROLLER2;  		 //!< qspi reg  :31
 volatile uint32_t QSPI_AES_KEY_0_3;   						 //!< qspi reg
 volatile uint32_t QSPI_AES_KEY_4_7;   						 //!< qspi reg
 volatile uint32_t QSPI_AES_KEY_8_B;   						 //!< qspi reg
 volatile uint32_t QSPI_AES_KEY_C_F;  						 //!< qspi reg
 volatile uint32_t QSPI_AES_NONCE_0_3;   					 //!< qspi reg
 volatile uint32_t QSPI_AES_NONCE_4_7;   					 //!< qspi reg
 volatile uint32_t QSPI_AES_NONCE_8_B;   					 //!< qspi reg
 volatile uint32_t QSPI_AES_SEC_SEG_ADDR[4 * 2];   //!< qspi reg
 volatile uint32_t RESERVED_6[6];   							 //!< qspi reg     
 volatile uint32_t QSPI_SEMI_AUTO_ADDR_REG;
 volatile uint32_t QSPI_SEMI_AUTO_MODE_CONFIG_REG;
 volatile uint32_t QSPI_SEMI_AUTO_MODE_CONFIG2_REG;
 volatile uint32_t QSPI_BUS_MODE2_REG;
 volatile uint32_t QSPI_AES_SEC_KEY_FRM_KH;
 volatile uint32_t QSPI_AUTO_CONITNUE_FETCH_CTRL_REG;   //!< qspi reg     
};

typedef struct spi_config_1_s {
	uint32_t inst_mode : 2;          //!< instruction will be sent in this mode
	uint32_t addr_mode : 2;          //!< addr will be sent in this mode
	uint32_t data_mode : 2;          //!< data will be sent/received in this mode
	uint32_t dummy_mode : 2;         //!< dummy bytes will be sent/received in this mode
	uint32_t extra_byte_mode : 2;    //!< extra bytes will be sent in this mode
	uint32_t prefetch_en : 1;     	 //!< prefetch enable
	uint32_t dummy_W_or_R : 1;       //!< dummy writes or read select
	uint32_t extra_byte_en : 1;      //!< Enable extra byte
	uint32_t d3d2_data : 2;          //!< Data on D3 and D2 line in SPI or DUAL mode
	uint32_t continuous : 1;         //!< continuous mode select
	uint32_t read_cmd  : 8;          //!< read cmd to be used
	uint32_t flash_type : 4;         //!< flash defines
	uint32_t no_of_dummy_bytes : 4;  //!< no_of_dummy_bytes to be used for read operations
} spi_config_1_t;

//! This structure members are used to configure qspi
typedef struct spi_config_2_s {
	uint32_t auto_mode : 1;         //!< mode select
	uint32_t cs_no : 2;             //!<  QSPI chip_select
	uint32_t reserved1 : 1;         //!< Jump Enable
	uint32_t neg_edge_sampling : 1; //!< For High speed mode, sample at neg edge
	uint32_t qspi_clk_en : 1;       //!< qspi clk select
	uint32_t protection : 2;        //!< flash protection select
	uint32_t dma_mode : 1;          //!< dma mode enable
	uint32_t swap_en : 1;           //!< swap enable for w/r
	uint32_t full_duplex : 2;       //!< full duplex mode select
	uint32_t wrap_len_in_bytes : 3; //!< wrap len to be used
  uint32_t addr_width_valid : 1;         
	uint32_t addr_width : 3;        //!< addr width to used
  uint32_t dummy_cycles_for_controller : 2;
  uint32_t reserved2 : 6;
	uint32_t pinset_valid : 1;
  uint32_t flash_pinset : 4;    //!< width of memory protection reg for sst flashes
} spi_config_2_t;

//! This structure members are used to configure qspi
typedef struct spi_config_3_s {
  uint32_t en_word_swap : 1;
  uint32_t _16bit_cmd_valid : 1;
  uint32_t _16bit_rd_cmd_msb : 8;
  uint32_t xip_mode          : 1;
  uint32_t no_of_dummy_bytes_wrap : 4; //!< no_of_dummy_bytes to be used for wrap operations 
  uint32_t ddr_mode_en : 1;
  uint32_t wr_cmd   : 8;
  uint32_t wr_inst_mode   : 2;
  uint32_t wr_addr_mode   : 2;
  uint32_t wr_data_mode   : 2;
  uint32_t dummys_4_jump : 2;     //!< no_of_dummy_bytes in case of jump instruction
} spi_config_3_t;

typedef struct spi_config_4_s {
  uint32_t _16bit_wr_cmd_msb : 8;
  uint32_t qspi_ddr_clk_en : 1;
  uint32_t qspi_loop_back_mode_en : 1;
  uint32_t qspi_manual_ddr_phasse : 1;
  uint32_t ddr_data_mode : 1;
  uint32_t ddr_inst_mode : 1;
  uint32_t ddr_addr_mode : 1;
  uint32_t ddr_dummy_mode : 1;
  uint32_t ddr_extra_byte : 1;
  uint32_t dual_flash_mode : 1;
  uint32_t secondary_csn : 1 ;
  uint32_t polarity_mode : 1 ;
  uint32_t valid_prot_bits : 4;
  uint32_t no_of_ms_dummy_bytes : 4;
	uint32_t ddr_dll_en :1;
  uint32_t continue_fetch_en : 1;
  uint32_t dma_write : 1;
  uint32_t prot_top_bottom : 1;
  uint32_t auto_csn_based_addr_en : 1;
} spi_config_4_t;
typedef struct spi_config_5_s {
   uint32_t block_erase_cmd: 16;
   uint32_t busy_bit_pos : 3;
   uint32_t d7_d4_data : 4;
   uint32_t dummy_bytes_for_rdsr : 4;
   uint32_t reset_type : 5;
} spi_config_5_t;

typedef struct spi_config_6_s {
   uint32_t chip_erase_cmd : 16;
   uint32_t sector_erase_cmd : 16;
} spi_config_6_t;

typedef struct spi_config_7_s {
   uint32_t status_reg_write_cmd : 16;
   uint32_t status_reg_read_cmd : 16;
} spi_config_7_t;

//! This structure has two daughter structures to configure qspi
typedef struct spi_config_s {
  spi_config_1_t spi_config_1;  //!< daughter structure 1
  spi_config_2_t spi_config_2;  //!< daughter structure 2 
  spi_config_3_t spi_config_3;  //!< daughter structure 3 
  spi_config_4_t spi_config_4;  //!< daughter structure 4 
  spi_config_5_t spi_config_5;  //!< daughter structure 5 
  spi_config_6_t spi_config_6;  //!< daughter structure 5 
  spi_config_7_t spi_config_7;  //!< daughter structure 5 
} spi_config_t;


typedef struct qspi_reg_s qspi_reg_t;

/** @brief SPI master ROM indirect function structure */
	
typedef PRE_PACK struct POST_PACK {
	
	void (*RSI_QSPI_WriteToFlash)(qspi_reg_t *qspi_reg, uint32_t len_in_bits,	uint32_t cmd_addr_data,uint32_t cs_no);
	void (*RSI_QSPI_SwitchQspi2)(qspi_reg_t *qspi_reg,uint32_t mode,uint32_t cs_no);
	uint32_t (*RSI_QSPI_WaitFlashStatusIdle)(qspi_reg_t *qspi_reg,	spi_config_t *spi_config,	uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_EnableStatusRegWrite)(qspi_reg_t *qspi_reg,	uint32_t flash_type, spi_config_t *spi_config,	uint32_t cs_no);
	void (*RSI_QSPI_StatusRegWrite)(qspi_reg_t *qspi_reg,	uint32_t write_value, spi_config_t *spi_config,uint32_t wr_reg_delay_ms);
	uint32_t (*RSI_QSPI_FlashRegRead)(qspi_reg_t *qspi_reg, uint8_t reg_read_cmd, uint32_t cs_no,  spi_config_t *spi_config);
	void (*RSI_QSPI_FlashRegWrite)(qspi_reg_t *qspi_reg,		uint32_t reg_write_cmd,		uint32_t reg_write_value,		uint32_t cs_no, uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_SetFlashMode)(qspi_reg_t *qspi_reg,	uint32_t data_mode,	uint32_t cs_no,	uint32_t ddr_en,	uint32_t flash_type);
	void (*RSI_QSPI_ConfigQflash4Read)(qspi_reg_t *qspi_reg,spi_config_t *spi_config,	uint32_t addr);
	void (*RSI_QSPI_ManualRead)(qspi_reg_t *qspi_reg,spi_config_t *spi_config,uint32_t addr,uint8_t *data,uint32_t hsize,	uint32_t len_in_bytes, uint32_t manual_udma_read, void *udmaHandle, void *rpdmaHandle);
	void (*RSI_QSPI_AutoInit)(qspi_reg_t *qspi_reg,spi_config_t *spi_config);
	void (*RSI_QSPI_AutoRead)(uint32_t cs_no, uint32_t addr, uint8_t *data, uint32_t hsize, uint32_t len_in_bytes, spi_config_t *spi_config, uint32_t dma_flags);
	void (*RSI_QSPI_FlashInit)(qspi_reg_t *qspi_reg, spi_config_t *spi_config, uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_SpiInit)(qspi_reg_t *qspi_reg,spi_config_t *spi_config,uint32_t flash_init_req,uint32_t wr_reg_delay_ms,		uint8_t fifo_thrsld);
	void (*RSI_QSPI_SpiErase)(qspi_reg_t *qspi_reg,spi_config_t *spi_config,uint32_t erase_cmd,uint32_t blk_sec_addr,uint32_t dis_hw_ctrl,		uint32_t wr_reg_delay_ms);
	uint32_t (*RSI_QSPI_SpiWrite)(qspi_reg_t *qspi_reg,spi_config_t *spi_config,uint32_t write_cmd,uint32_t addr,uint8_t  *data,uint32_t len_in_bytes,uint16_t page_size,	uint32_t hsize,uint32_t dis_hw_ctrl,uint32_t wr_reg_delay_ms,uint32_t check_en,uint32_t udma_enable,void *udmaHandle,void *rpdmaHandle);
	void (*RSI_QSPI_SpiRead)(qspi_reg_t *qspi_reg,	spi_config_t *spi_config,	uint32_t addr,uint8_t  *data,	uint32_t  hsize,	uint32_t len_in_bytes, uint32_t manual_udma_read, void *udmaHandle,  void *rpdmaHandle);
	void (*RSI_QSPI_Usleep)(uint32_t delay_us);    //!< function ptr for halting processor for delay (us) specified
	void (*RSI_QSPI_WriteBlockProtect)(qspi_reg_t *qspi_reg,	uint32_t protect,	uint32_t cs_no,	uint32_t num_prot_bytes,	uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_QspiLoadKey)(qspi_reg_t *qspi_reg,	uint32_t *key, uint32_t kh_enable);
	void (*RSI_QSPI_QspiLoadNonce)(qspi_reg_t *qspi_reg, uint32_t *nonce);
	void (*RSI_QSPI_SegSecEn)(qspi_reg_t *qspi_reg,	uint32_t seg_no,	uint32_t start_addr,	uint32_t end_addr);
	void (*RSI_QSPI_StatusControlRegWrite)(spi_config_t * spi_config,	qspi_reg_t *qspi_reg,	uint16_t write_command,	uint32_t addr,	uint16_t write_value,	uint32_t cs_no, uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_FlashProtection)(spi_config_t *spi_config,qspi_reg_t *qspi_reg,		uint32_t protection, uint32_t wr_reg_delay_ms);
	void (*RSI_QSPI_ConfigQspiDll)(spi_config_t *spi_config, qspi_reg_t *qspi_reg);
  void (*RSI_QSPI_ResetFlash)(qspi_reg_t *qspi_reg, uint32_t cs_no);
  void (*RSI_QSPI_UpdateOperatingMode_and_ResetType)(qspi_reg_t *qspi_reg, uint32_t operating_mode);
 
}ROM_QSPI_API_T;

/* Pointer to QSPI peripheral driver functions in ROM */
#define ROMAPI_QSPI_API    	        ((ROM_QSPI_API_T *) (0x003002A0))

#ifdef __cplusplus
}
#endif

#endif /*__RSI_ROM_QSPI_H__*/
